@@ -37,8 +37,8 @@
 
 #include <sys/signalfd.h>
 #if defined(ANDROID)
+#include <sys/prctl.h>
 #include <sys/capability.h>
-#include <linux/prctl.h>
 #endif
 
 #include <glib.h>
@@ -86,7 +86,10 @@ static void service_register(const void *buf, uint16_t len)
 
 	switch (m->service_id) {
 	case HAL_SERVICE_ID_BLUETOOTH:
-		bt_bluetooth_register(hal_ipc, m->mode);
+		if (!bt_bluetooth_register(hal_ipc, m->mode)) {
+			status = HAL_STATUS_FAILED;
+			goto failed;
+		}
 
 		break;
 	case HAL_SERVICE_ID_SOCKET:
@@ -198,8 +201,10 @@ static void service_unregister(const void *buf, uint16_t len)
 		bt_health_unregister();
 		break;
 	default:
-		/* This would indicate bug in HAL, as unregister should not be
-		 * called in init failed */
+		/*
+		 * This would indicate bug in HAL, as unregister should not be
+		 * called in init failed
+		 */
 		DBG("service %u not supported", m->service_id);
 		status = HAL_STATUS_FAILED;
 		goto failed;
@@ -377,7 +382,7 @@ static void cleanup_services(void)
 
 	DBG("");
 
-	for (i = HAL_SERVICE_ID_BLUETOOTH; i < HAL_SERVICE_ID_MAX; i++) {
+	for (i = HAL_SERVICE_ID_MAX; i > HAL_SERVICE_ID_CORE; i--) {
 		if (!services[i])
 			continue;
 
@@ -424,9 +429,11 @@ static bool set_capabilities(void)
 	header.version = _LINUX_CAPABILITY_VERSION;
 	header.pid = 0;
 
-	/* CAP_NET_ADMIN: Allow use of MGMT interface
+	/*
+	 * CAP_NET_ADMIN: Allow use of MGMT interface
 	 * CAP_NET_BIND_SERVICE: Allow use of privileged PSM
-	 * CAP_NET_RAW: Allow use of bnep ioctl calls */
+	 * CAP_NET_RAW: Allow use of bnep ioctl calls
+	 */
 	cap.effective = cap.permitted =
 		CAP_TO_MASK(CAP_NET_RAW) |
 		CAP_TO_MASK(CAP_NET_ADMIN) |
