@@ -622,6 +622,7 @@ static void cmd_show_item(int argc, char *argv[])
 
 	rl_printf("Item %s\n", g_dbus_proxy_get_path(proxy));
 
+	print_property(proxy, "Player");
 	print_property(proxy, "Name");
 	print_property(proxy, "Type");
 	print_property(proxy, "FolderType");
@@ -737,6 +738,11 @@ static void cmd_change_folder(int argc, char *argv[])
 		return;
 	}
 
+	if (dbus_validate_path(argv[1], NULL) == FALSE) {
+		rl_printf("Not a valid path\n");
+		return;
+	}
+
 	if (check_default_player() == FALSE)
 		return;
 
@@ -1079,6 +1085,7 @@ static void rl_handler(char *input)
 	if (!strlen(input))
 		goto done;
 
+	g_strstrip(input);
 	add_history(input);
 
 	argv = g_strsplit(input, " ", -1);
