@@ -44,6 +44,7 @@
 #include <bluetooth/hci_lib.h>
 
 #include "src/textfile.h"
+#include "src/shared/util.h"
 #include "tools/csr.h"
 
 static struct hci_dev_info di;
@@ -75,12 +76,6 @@ static void print_dev_list(int ctl, int flags)
 		di.dev_id = (dr+i)->dev_id;
 		if (ioctl(ctl, HCIGETDEVINFO, (void *) &di) < 0)
 			continue;
-		if (hci_test_bit(HCI_RAW, &di.flags) &&
-				!bacmp(&di.bdaddr, BDADDR_ANY)) {
-			int dd = hci_open_dev(di.dev_id);
-			hci_read_bd_addr(dd, &di.bdaddr, 1000);
-			hci_close_dev(dd);
-		}
 		print_dev_info(ctl, &di);
 	}
 }
@@ -855,6 +850,7 @@ static char *get_minor_device_name(int major, int minor)
 		}
 		if (strlen(cls_str) > 0)
 			return cls_str;
+		break;
 	}
 	case 6:	/* imaging */
 		if (minor & 4)
@@ -1316,7 +1312,7 @@ static void cmd_inq_data(int ctl, int hdev, char *opt)
 				printf("\t%s service classes:",
 					type == 0x02 ? "Shortened" : "Complete");
 				for (i = 0; i < (len - 1) / 2; i++) {
-					uint16_t val = bt_get_le16((ptr + (i * 2)));
+					uint16_t val = get_le16((ptr + (i * 2)));
 					printf(" 0x%4.4x", val);
 				}
 				printf("\n");
@@ -2015,13 +2011,6 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 
-	if (hci_test_bit(HCI_RAW, &di.flags) &&
-			!bacmp(&di.bdaddr, BDADDR_ANY)) {
-		int dd = hci_open_dev(di.dev_id);
-		hci_read_bd_addr(dd, &di.bdaddr, 1000);
-		hci_close_dev(dd);
-	}
-
 	while (argc > 0) {
 		for (i = 0; command[i].cmd; i++) {
 			if (strncmp(command[i].cmd,
